<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="krecipes" version="10">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="reload_action" />
    <Separator />
    <Action name="import_action" />
    <Action name="import_db_action" />
    <Action name="export_action" />
    <Separator />
    <Action name="backup_action" />
    <Action name="restore_action" />
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Action name="page_setup_action" />
    <Action name="print_setup_action" />
    <Separator />
    <Action name="copy_to_clipboard_action" />
</Menu>
  <Menu name="tools"><text>&amp;Tools</text>
    <Action name="edit_action" />
    <Action name="converter_action" />
    <Action name="merge_categories_action" />
    <Action name="merge_ingredients_action" />
</Menu>
</MenuBar>
</kpartgui>
